=== modified file 'NEWS'
--- NEWS	2008-06-11 12:03:48 +0000
+++ NEWS	2008-06-17 01:29:47 +0000
@@ -8,8 +8,6 @@
 IN DEVELOPMENT
 --------------
 
-  BUG FIXES:
-
   IMPROVEMENTS:
 
     * ``bzr export --format=tgz --root=NAME -`` to export a gzipped tarball 
@@ -18,14 +16,32 @@
 
   BUGFIXES:
 
+    * Clearer message about how to set the PYTHONPATH if bzrlib can't be
+      loaded. 
+      (Martin Pool, #205230)
+
+    * Errors about missing libraries are now shown without a traceback,
+      and with a suggestion to install the library.  The full traceback is 
+      still in ``.bzr.log`` and can be shown with ``-Derror``.
+      (Martin Pool, #240161)
+
+    * ``needs_read_lock`` and ``needs_write_lock`` now suppress an error during
+      ``unlock`` if there was an error in the original function. This helps
+      most when there is a failure with a smart server action, since often the
+      connection closes and we cannot unlock.
+      (Andrew Bennetts, John Arbash Meinel, #125784)
+
+    * Raise the correct exception when doing ``-rbefore:0`` or ``-c0``.
+      (John Arbash Meinel, #239933)
+
     * You can now compare file revisions in Windows diff programs from 
       Cygwin Bazaar.
       (Matt McClure, #209281)
 
   DOCUMENTATION:
 
-   * Updated developer documentation.
-     (Martin Pool)
+    * Updated developer documentation.
+      (Martin Pool)
 
   TESTING:
 
@@ -49,6 +65,9 @@
       reading from the repository.)
       (Martin Pool, Andrew Bennetts, Robert Collins, #234748)
 
+    * ``bzr status`` was breaking if you merged the same revision twice.
+      (John Arbash Meinel, #235407)
+
     * Fix infinite loop consuming 100% CPU when a connection is lost while
       reading a response body via the smart protocol v1 or v2.
       (Andrew Bennetts)
@@ -81,11 +100,6 @@
     * Faster branching (1.09x) and lightweight checkouts (1.06x) on large trees.
       (Ian Clatworthy, Aaron Bentley)
 
-  BUGFIXES:
-
-    * ``bzr status`` was breaking if you merged the same revision twice.
-      (John Arbash Meinel, #235407)
-
   DOCUMENTATION:
 
     * Added *Bazaar Zen* section to the User Guide. (Ian Clatworthy)

=== modified file 'bzr'
--- bzr	2008-05-10 11:33:32 +0000
+++ bzr	2008-06-17 01:29:47 +0000
@@ -64,9 +64,9 @@
     import bzrlib
 except ImportError, e:
     sys.stderr.write("bzr: ERROR: "
-                     "Couldn't import bzrlib and dependencies.\n"
-                     "Please check bzrlib is on your PYTHONPATH.\n"
-                     "\n")
+        "Couldn't import bzrlib and dependencies.\n"
+        "Please check the directory containing bzrlib is on your PYTHONPATH.\n"
+        "\n")
     raise
 
 if bzrlib.version_info[:3] != _script_version:

=== modified file 'bzrlib/decorators.py'
--- bzrlib/decorators.py	2007-01-25 15:54:29 +0000
+++ bzrlib/decorators.py	2008-06-08 17:00:25 +0000
@@ -22,6 +22,9 @@
            ]
 
 
+import sys
+
+
 def _get_parameters(func):
     """Recreate the parameters for a function using introspection.
 
@@ -89,9 +92,17 @@
 def %(name)s_read_locked(%(params)s):
     self.lock_read()
     try:
-        return unbound(%(passed_params)s)
-    finally:
+        result = unbound(%(passed_params)s)
+    except:
+        import sys
+        exc_info = sys.exc_info()
+        try:
+            self.unlock()
+        finally:
+            raise exc_info[0], exc_info[1], exc_info[2]
+    else:
         self.unlock()
+        return result
 read_locked = %(name)s_read_locked
 """
     params, passed_params = _get_parameters(unbound)
@@ -124,9 +135,17 @@
     def read_locked(self, *args, **kwargs):
         self.lock_read()
         try:
-            return unbound(self, *args, **kwargs)
-        finally:
+            result = unbound(self, *args, **kwargs)
+        except:
+            import sys
+            exc_info = sys.exc_info()
+            try:
+                self.unlock()
+            finally:
+                raise exc_info[0], exc_info[1], exc_info[2]
+        else:
             self.unlock()
+            return result
     read_locked.__doc__ = unbound.__doc__
     read_locked.__name__ = unbound.__name__
     return read_locked
@@ -138,9 +157,17 @@
 def %(name)s_write_locked(%(params)s):
     self.lock_write()
     try:
-        return unbound(%(passed_params)s)
-    finally:
+        result = unbound(%(passed_params)s)
+    except:
+        import sys
+        exc_info = sys.exc_info()
+        try:
+            self.unlock()
+        finally:
+            raise exc_info[0], exc_info[1], exc_info[2]
+    else:
         self.unlock()
+        return result
 write_locked = %(name)s_write_locked
 """
     params, passed_params = _get_parameters(unbound)
@@ -162,9 +189,16 @@
     def write_locked(self, *args, **kwargs):
         self.lock_write()
         try:
-            return unbound(self, *args, **kwargs)
-        finally:
+            result = unbound(self, *args, **kwargs)
+        except:
+            exc_info = sys.exc_info()
+            try:
+                self.unlock()
+            finally:
+                raise exc_info[0], exc_info[1], exc_info[2]
+        else:
             self.unlock()
+            return result
     write_locked.__doc__ = unbound.__doc__
     write_locked.__name__ = unbound.__name__
     return write_locked

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2008-05-30 00:42:24 +0000
+++ bzrlib/revisionspec.py	2008-06-15 23:08:14 +0000
@@ -522,7 +522,7 @@
         base_revspec = RevisionSpec.from_string(self.spec)
         base_revision_id = base_revspec.as_revision_id(context_branch)
         if base_revision_id == revision.NULL_REVISION:
-            raise errors.InvalidRevisionSpec(self.user_spec, branch,
+            raise errors.InvalidRevisionSpec(self.user_spec, context_branch,
                                          'cannot go before the null: revision')
         context_repo = context_branch.repository
         context_repo.lock_read()

=== modified file 'bzrlib/tests/test_decorators.py'
--- bzrlib/tests/test_decorators.py	2007-01-24 17:59:50 +0000
+++ bzrlib/tests/test_decorators.py	2008-06-08 17:00:25 +0000
@@ -20,37 +20,139 @@
 import inspect
 
 from bzrlib import decorators
-from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.tests import TestCase
 
 
-class DecoratorSample(object):
-    """Sample class that uses decorators.
+def create_decorator_sample(style, except_in_unlock=False):
+    """Create a DecoratorSample object, using specific lock operators.
 
-    Log when requests go through lock_read()/unlock() or lock_write()/unlock.
+    :param style: The type of lock decorators to use (fast/pretty/None)
+    :param except_in_unlock: If True, raise an exception during unlock
+    :return: An instantiated DecoratorSample object.
     """
 
-    def __init__(self):
-        self.actions = []
-
-    def lock_read(self):
-        self.actions.append('lock_read')
-
-    def lock_write(self):
-        self.actions.append('lock_write')
-
-    def unlock(self):
-        self.actions.append('unlock')
-
-    @needs_read_lock
-    def frob(self):
-        """Frob the sample object"""
-        self.actions.append('frob')
-
-    @needs_write_lock
-    def bank(self, bar, biz=None):
-        """Bank the sample, but using bar and biz."""
-        self.actions.append(('bank', bar, biz))
+    if style is None:
+        # Default
+        needs_read_lock = decorators.needs_read_lock
+        needs_write_lock = decorators.needs_write_lock
+    elif style == 'pretty':
+        needs_read_lock = decorators._pretty_needs_read_lock
+        needs_write_lock = decorators._pretty_needs_write_lock
+    else:
+        needs_read_lock = decorators._fast_needs_read_lock
+        needs_write_lock = decorators._fast_needs_write_lock
+
+    class DecoratorSample(object):
+        """Sample class that uses decorators.
+
+        Log when requests go through lock_read()/unlock() or
+        lock_write()/unlock.
+        """
+
+        def __init__(self):
+            self.actions = []
+
+        def lock_read(self):
+            self.actions.append('lock_read')
+
+        def lock_write(self):
+            self.actions.append('lock_write')
+
+        def unlock(self):
+            if except_in_unlock:
+                self.actions.append('unlock_fail')
+                raise KeyError('during unlock')
+            else:
+                self.actions.append('unlock')
+
+        @needs_read_lock
+        def frob(self):
+            """Frob the sample object"""
+            self.actions.append('frob')
+            return 'newbie'
+
+        @needs_write_lock
+        def bank(self, bar, biz=None):
+            """Bank the sample, but using bar and biz."""
+            self.actions.append(('bank', bar, biz))
+            return (bar, biz)
+
+        @needs_read_lock
+        def fail_during_read(self):
+            self.actions.append('fail_during_read')
+            raise TypeError('during read')
+
+        @needs_write_lock
+        def fail_during_write(self):
+            self.actions.append('fail_during_write')
+            raise TypeError('during write')
+
+    return DecoratorSample()
+
+
+class TestDecoratorActions(TestCase):
+
+    _decorator_style = None # default
+
+    def test_read_lock_locks_and_unlocks(self):
+        sam = create_decorator_sample(self._decorator_style)
+        self.assertEqual('newbie', sam.frob())
+        self.assertEqual(['lock_read', 'frob', 'unlock'], sam.actions)
+
+    def test_write_lock_locks_and_unlocks(self):
+        sam = create_decorator_sample(self._decorator_style)
+        self.assertEqual(('bar', 'bing'), sam.bank('bar', biz='bing'))
+        self.assertEqual(['lock_write', ('bank', 'bar', 'bing'), 'unlock'],
+                         sam.actions)
+
+    def test_read_lock_unlocks_during_failure(self):
+        sam = create_decorator_sample(self._decorator_style)
+        self.assertRaises(TypeError, sam.fail_during_read)
+        self.assertEqual(['lock_read', 'fail_during_read', 'unlock'],
+                         sam.actions)
+
+    def test_write_lock_unlocks_during_failure(self):
+        sam = create_decorator_sample(self._decorator_style)
+        self.assertRaises(TypeError, sam.fail_during_write)
+        self.assertEqual(['lock_write', 'fail_during_write', 'unlock'],
+                         sam.actions)
+
+    def test_read_lock_raises_original_error(self):
+        sam = create_decorator_sample(self._decorator_style,
+                                      except_in_unlock=True)
+        self.assertRaises(TypeError, sam.fail_during_read)
+        self.assertEqual(['lock_read', 'fail_during_read', 'unlock_fail'],
+                         sam.actions)
+
+    def test_write_lock_raises_original_error(self):
+        sam = create_decorator_sample(self._decorator_style,
+                                      except_in_unlock=True)
+        self.assertRaises(TypeError, sam.fail_during_write)
+        self.assertEqual(['lock_write', 'fail_during_write', 'unlock_fail'],
+                         sam.actions)
+
+    def test_read_lock_raises_unlock_error(self):
+        sam = create_decorator_sample(self._decorator_style,
+                                      except_in_unlock=True)
+        self.assertRaises(KeyError, sam.frob)
+        self.assertEqual(['lock_read', 'frob', 'unlock_fail'], sam.actions)
+
+    def test_write_lock_raises_unlock_error(self):
+        sam = create_decorator_sample(self._decorator_style,
+                                      except_in_unlock=True)
+        self.assertRaises(KeyError, sam.bank, 'bar', biz='bing')
+        self.assertEqual(['lock_write', ('bank', 'bar', 'bing'),
+                          'unlock_fail'], sam.actions)
+
+
+class TestFastDecoratorActions(TestDecoratorActions):
+
+    _decorator_style = 'fast'
+
+
+class TestPrettyDecoratorActions(TestDecoratorActions):
+
+    _decorator_style = 'pretty'
 
 
 class TestDecoratorDocs(TestCase):
@@ -58,20 +160,20 @@
 
     def test_read_lock_passthrough(self):
         """@needs_read_lock exposes underlying name and doc."""
-        sam = DecoratorSample()
+        sam = create_decorator_sample(None)
         self.assertEqual('frob', sam.frob.__name__)
         self.assertEqual('Frob the sample object', sam.frob.__doc__)
 
     def test_write_lock_passthrough(self):
         """@needs_write_lock exposes underlying name and doc."""
-        sam = DecoratorSample()
+        sam = create_decorator_sample(None)
         self.assertEqual('bank', sam.bank.__name__)
         self.assertEqual('Bank the sample, but using bar and biz.',
                          sam.bank.__doc__)
 
     def test_argument_passthrough(self):
         """Test that arguments get passed around properly."""
-        sam = DecoratorSample()
+        sam = create_decorator_sample(None)
         sam.bank('1', biz='2')
         self.assertEqual(['lock_write',
                           ('bank', '1', '2'),

=== modified file 'bzrlib/tests/test_revisionspec.py'
--- bzrlib/tests/test_revisionspec.py	2008-05-30 01:03:18 +0000
+++ bzrlib/tests/test_revisionspec.py	2008-06-16 16:13:26 +0000
@@ -73,15 +73,26 @@
                          ' %r != %r'
                          % (revision_spec, exp_revision_id, rev_info.rev_id))
 
-    def assertInvalid(self, revision_spec, extra=''):
+    def assertInvalid(self, revision_spec, extra='',
+                      invalid_as_revision_id=True):
         try:
             self.get_in_history(revision_spec)
         except errors.InvalidRevisionSpec, e:
             self.assertEqual(revision_spec, e.spec)
             self.assertEqual(extra, e.extra)
         else:
-            self.fail('Expected InvalidRevisionSpec to be raised for %s'
-                      % (revision_spec,))
+            self.fail('Expected InvalidRevisionSpec to be raised for'
+                      ' %r.in_history' % (revision_spec,))
+        if invalid_as_revision_id:
+            try:
+                spec = RevisionSpec.from_string(revision_spec)
+                spec.as_revision_id(self.tree.branch)
+            except errors.InvalidRevisionSpec, e:
+                self.assertEqual(revision_spec, e.spec)
+                self.assertEqual(extra, e.extra)
+            else:
+                self.fail('Expected InvalidRevisionSpec to be raised for'
+                          ' %r.as_revision_id' % (revision_spec,))
 
     def assertAsRevisionId(self, revision_id, revision_spec):
         """Calling as_revision_id() should return the specified id."""
@@ -299,7 +310,7 @@
         self.assertInHistoryIs(2, 'r2', 'revid:r2')
         
     def test_missing(self):
-        self.assertInvalid('revid:r3')
+        self.assertInvalid('revid:r3', invalid_as_revision_id=False)
 
     def test_merged(self):
         """We can reach revisions in the ancestry"""
@@ -308,7 +319,7 @@
     def test_not_here(self):
         self.tree2.commit('alt third', rev_id='alt_r3')
         # It exists in tree2, but not in tree
-        self.assertInvalid('revid:alt_r3')
+        self.assertInvalid('revid:alt_r3', invalid_as_revision_id=False)
 
     def test_in_repository(self):
         """We can get any revision id in the repository"""

=== modified file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	2008-02-07 03:49:57 +0000
+++ bzrlib/tests/test_trace.py	2008-06-17 01:13:49 +0000
@@ -93,6 +93,28 @@
         self.assertTrue(len(msg) > 0)
         self.assertEqualDiff(msg, 'bzr: ERROR: Not a branch: \"wibble\".\n')
 
+    def test_report_external_import_error(self):
+        """Short friendly message for missing system modules."""
+        try:
+            import ImaginaryModule
+        except ImportError, e:
+            pass
+        else:
+            self.fail("somehow succeeded in importing %r" % ImaginaryModule)
+        msg = _format_exception()
+        self.assertEqual(msg,
+            'bzr: ERROR: No module named ImaginaryModule\n'
+            'You may need to install this Python library separately.\n')
+
+    def test_report_import_syntax_error(self):
+        try:
+            raise ImportError("syntax error")
+        except ImportError, e:
+            pass
+        msg = _format_exception()
+        self.assertContainsRe(msg,
+            r"Traceback \(most recent call last\)")
+
     def test_trace_unicode(self):
         """Write Unicode to trace log"""
         self.log(u'the unicode character for benzene is \N{BENZENE RING}')

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2008-05-15 17:24:12 +0000
+++ bzrlib/trace.py	2008-06-17 01:13:49 +0000
@@ -401,6 +401,11 @@
     elif isinstance(exc_object, KeyboardInterrupt):
         err_file.write("bzr: interrupted\n")
         return errors.EXIT_ERROR
+    elif isinstance(exc_object, ImportError) \
+        and str(exc_object).startswith("No module named "):
+        report_user_error(exc_info, err_file,
+            'You may need to install this Python library separately.')
+        return errors.EXIT_ERROR
     elif not getattr(exc_object, 'internal_error', True):
         report_user_error(exc_info, err_file)
         return errors.EXIT_ERROR
@@ -423,15 +428,21 @@
 
 
 # TODO: Should these be specially encoding the output?
-def report_user_error(exc_info, err_file):
+def report_user_error(exc_info, err_file, advice=None):
     """Report to err_file an error that's not an internal error.
 
     These don't get a traceback unless -Derror was given.
+
+    :param exc_info: 3-tuple from sys.exc_info()
+    :param advice: Extra advice to the user to be printed following the
+        exception.
     """
     if 'error' in debug.debug_flags:
         print_exception(exc_info, err_file)
         return
     err_file.write("bzr: ERROR: %s\n" % (exc_info[1],))
+    if advice:
+        err_file.write("%s\n" % (advice,))
 
 
 def report_bug(exc_info, err_file):

=== modified file 'doc/developers/releasing.txt'
--- doc/developers/releasing.txt	2008-06-11 02:28:10 +0000
+++ doc/developers/releasing.txt	2008-06-17 00:18:39 +0000
@@ -239,6 +239,8 @@
 #. For final releases, also send the announcement mail to
    info-gnu@gnu.org and python-announce-list@python.org.
 
+#. Also send a GNU directory update to bug-directory@gnu.org.
+
 #. Update the python package index: <http://pypi.python.org/pypi/bzr> - best
    done by running ::
 

